@@ -49,6 +49,7 @@ import urllib
 import zipfile
 
 import application
+import errors
 import install
 import mozinfo
 import mozmill
@@ -71,13 +72,6 @@ MOZMILL_RESTART_CLI = {
     'thunderbird' : mozmill.ThunderbirdRestartCLI,
 }
 
-
-class TestFailedException(Exception):
-    """Exception for a resource not being found (e.g. no logs)"""
-    def __init__(self):
-        Exception.__init__(self, "Some tests have been failed.")
-
-
 class TestRun(object):
     """ Class to execute a Mozmill test-run. """
 
@@ -298,6 +292,21 @@ class TestRun(object):
         except Exception, e:
             print e
 
+    def get_tests_folder(self, testrun_id, *args):
+        """ Getting the correct tests path for the testrun. """
+
+        app_path = os.path.join(self.repository.path, self.options.application)
+        if os.path.isdir(app_path):
+            # Check if the application supports this testrun
+            path = os.path.join(app_path, 'tests', testrun_id, *args)
+            if not os.path.isdir(path):
+                raise errors.NotSupportedTestrunException(self)
+        # TODO: Remove this else block once we get the new repository structure landed
+        else:
+            path = os.path.join(self.repository.path, 'tests', testrun_id, *args)
+
+        return path
+
     def prepare_addons(self):
         """ Prepare the addons for the test run. """
 
@@ -450,7 +459,7 @@ class TestRun(object):
 
             # If a test has been failed ensure that we exit with status 2
             if self.last_failed_tests:
-                raise TestFailedException()
+                raise errors.TestFailedException()
 
     def send_report(self, report_url):
         """ Send the report to a CouchDB instance """
@@ -502,7 +511,7 @@ class AddonsTestRun(TestRun):
     def get_all_addons(self):
         """ Retrieves all add-ons inside the "addons" folder. """
 
-        path = os.path.join(self.repository.path, "tests", "addons")
+        path = TestRun.get_tests_folder(self, 'addons')
         return [entry for entry in os.listdir(path)
                       if os.path.isdir(os.path.join(path, entry))]
 
@@ -540,7 +549,9 @@ class AddonsTestRun(TestRun):
                 self.target_addon = None
 
                 # Get the download URL
-                self._addon_path = os.path.join('tests', 'addons', self._addon)
+                self._addon_path = TestRun.get_tests_folder(self, 'addons',
+                                                            self._addon)
+
                 url = self.get_download_url()
 
                 if url is None:
@@ -573,6 +584,7 @@ class AddonsTestRun(TestRun):
                 self.test_path = os.path.join(self._addon_path, 'restartTests')
                 if os.path.isdir(os.path.join(self.repository.path, self.test_path)):
                     try:
+                        print "Attempting to run restart tests"
                         self.restart_tests = True
                         self.addon_list.append(self.target_addon)
                         TestRun.run_tests(self)
@@ -673,9 +685,12 @@ class EnduranceTestRun(TestRun):
         self.endurance_results = []
 
         try:
-            self.test_path = os.path.join('tests', 'endurance')
+            self.test_path = TestRun.get_tests_folder(self, 'endurance')
+
             if self.options.reserved:
-                self.test_path = os.path.join(self.test_path, 'reserved', self.options.reserved)
+                self.test_path = os.path.join(self.test_path,
+                                              'reserved',
+                                              self.options.reserved)
             TestRun.run_tests(self)
 
         except Exception, e:
@@ -735,15 +750,18 @@ class FunctionalTestRun(TestRun):
 
         try:
             self.restart_tests = False
-            self.test_path = os.path.join('tests', 'functional')
+            self.test_path = TestRun.get_tests_folder(self, 'functional')
+
             TestRun.run_tests(self)
         except Exception, e:
             print str(e)
             self.last_exception = e
 
         try:
+            print "Attempting to run restart tests"
             self.restart_tests = True
-            self.test_path = os.path.join('tests', 'functional', 'restartTests')
+            self.test_path = TestRun.get_tests_folder(self, 'functional', 'restartTests')
+
             TestRun.run_tests(self)
         except Exception, e:
             print str(e)
@@ -764,7 +782,8 @@ class L10nTestRun(TestRun):
 
         try:
             self.restart_tests = True
-            self.test_path = os.path.join('tests','l10n')
+            self.test_path = TestRun.get_tests_folder(self, 'l10n')
+
             TestRun.run_tests(self)
         except Exception, e:
             print str(e)
@@ -785,15 +804,18 @@ class RemoteTestRun(TestRun):
 
         try:
             self.restart_tests = False
-            self.test_path = os.path.join('tests', 'remote')
+            self.test_path = TestRun.get_tests_folder(self, 'remote')
+
             TestRun.run_tests(self)
         except Exception, e:
             print str(e)
             self.last_exception = e
 
         try:
+            print "Attempting to run restart tests"
             self.restart_tests = True
-            self.test_path = os.path.join('tests', 'remote', 'restartTests')
+            self.test_path = TestRun.get_tests_folder(self, 'remote', 'restartTests')
+
             TestRun.run_tests(self)
         except Exception, e:
             print str(e)
@@ -950,7 +972,8 @@ class UpdateTestRun(TestRun):
     def run_update_tests(self, is_fallback):
         try:
             folder = 'testFallbackUpdate' if is_fallback else 'testDirectUpdate'
-            self.test_path = os.path.join('tests', 'update', folder)
+            self.test_path = TestRun.get_tests_folder(self, 'update', folder)
+
             TestRun.run_tests(self)
         except Exception, e:
             print "Execution of test-run aborted: %s" % str(e)
@@ -996,8 +1019,10 @@ class UpdateTestRun(TestRun):
 def exec_testrun(cls):
     try:
         cls().run()
-    except TestFailedException:
+    except errors.TestFailedException:
         sys.exit(2)
+    except errors.NotSupportedTestrunException:
+        sys.exit(3)
 
 
 def addons_cli():
